function PeakCaller

% PeakCaller -- this gui was designed by members of the Hussman Institute
% for Autism in order to automate the process of quantifying intracellular 
% Ca2+ transients in neuronal cultures. 
%
% This gui provides a number of options that allow the user to (1) remove
% a trend from the data and (2) describe the properties of the peaks that
% are of interest to them. 
%
% TREND REMOVAL: Before looking for peaks, the data may be smoothed using
% either  
%   1. a one-sided or two-sided exponential moving average (EMA)
%   2. a finite-difference diffusion
%   3. a convex envelope
%
% PEAK PROPERTIES: For a chosen file containing the data for several
% regions of interest (ROIs), the user can specify what a peak should look
% like, including: 
%   1. The required rise before a peak
%   2. The required fall after a peak
%   3. The maximum window before a peak in which the rise must occur
%   4. The maximum window after a peak in which a fall must occur.
%
% ADDITIONAL OPTIONS: The user may also specify whether to 
%   1. Try and recover signals ruined by inappropriate drop offs due to bad
%      shutter timing. 
%   2. Compare PeakCaller's findings against the "old" PeakFinder code.
%
% OUTPUT OPPORTUNITIES: The GUI has a variety options for visualizing the
% results, including:
%   1. Scrolling through the individual traces and POPPING the current
%      trace for easier manipulation and formatting.
%   2. Viewing all of the traces simultaneously in a single plot.
%   3. Viewing a raster plot that highlights the peaks across the various
%      ROIs.
%   4. Viewing a pair of histograms that visually summarize the properties
%      of the peaks.
%   5. Looking at a heat map showing the maximum lagged correlations of the
%      different ROIs.
%   6. Looking at a heat map showing the synchronization index of the
%      different ROIs.
%   7. Exporting the data for the various peaks to an Excel spreadsheet.

%%

% Create a figure for PeakCaller -- this is the 'world' this GUI lives in...

scrsz = get(0,'ScreenSize');

figure_width = min(1280,scrsz(3)-40);
figure_height = min(720,scrsz(4)-120);

h.figure = figure( ...
    'Units', 'pixels',...
    'NumberTitle', 'off', ...
    'Menubar', 'none', ...
    'Toolbar', 'none', ...
    'Name', 'PeakCaller (hussmanautism.org)', ...
    'Position', [20 scrsz(4)-figure_height-100 figure_width figure_height], ...
    'Color', [0.9 0.9 0.9]); clf;

% The left-hand side of the figure is composed of two axes -- a plot of the
% original calcium signal and a plot of the trend-adjusted signal....

h.original_axes = axes(...
    'Units', 'normalized',...
    'Position', [0.06 0.59 0.60 0.36],...
    'FontName', 'Cambria',...
    'Fontsize', 8);

h.adjusted_axes = axes(...
    'Units', 'normalized',...
    'Position', [0.06 0.18 0.60 0.36],...
    'FontName', 'Cambria',...
    'Fontsize', 8);

% Below the plots is a slider that will allow the user to scroll through
% the various signals....

h.series_slider = uicontrol(...
    'Style', 'slider',...
    'Units', 'normalized',...
    'Position', [0.06 0.07 0.60 0.04],...
    'Min', 1, ...
    'Max', 5,...
    'SliderStep', [0.25,0.25],...
    'BackgroundColor', [0.5 0.5 0.5],...
    'Visible', 'off',...
    'Value', 1);

h.series_text = uicontrol(...
    'Style', 'text',...
    'Units', 'normalized',...
    'FontName', 'Cambria',...
    'Fontsize', 14,...
    'HorizontalAlignment', 'Center',...
    'Position', [0.30 0.02 0.12 0.04],...
    'BackgroundColor', [0.9 0.9 0.9],...
    'Visible', 'off',...
    'String', '1');

% The right-hand side of the figure is made up of the options for choosing
% a file, setting the parameters that drive PeakCaller, and viewing the
% results.... 

% First off, there is a button that opens a file menu, allowing the user to
% decide which data to use for the test.

h.file_panel = uipanel(...
    'Units', 'normalized',...
    'Position', [0.69 0.88 0.28 0.07]);

h.file_textbox = uicontrol( ...
    'Parent', h.file_panel,...
    'Style', 'text', ...
    'Units', 'normalized', ... 
    'Position', [0.00 0.51 1.00 0.47], ...
    'FontName', 'Cambria',...
    'Fontsize', 11,...
    'HorizontalAlignment', 'center');

h.type_textbox = uicontrol( ...
    'Style', 'text', ...
    'Parent', h.file_panel,...
    'Units', 'normalized', ... 
    'Position', [0.00 0.02 1.00 0.47], ...
    'FontName', 'Cambria',...
    'Fontsize', 11,...
    'HorizontalAlignment', 'center');

h.file_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 16,...
    'BackgroundColor', [0.2 0.2 0.3],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.71 0.83 0.24 0.04],...
    'Interruptible', 'off',...
    'String', 'Choose File');

% Once a file is chosen, a panel opens up that displays all of the
% options the user has for defining peaks....

h.view_panel = uipanel(...
    'Units', 'normalized',...
    'Position', [0.69 0.23 0.28 0.58],...
    'Visible', 'off',...
    'FontName', 'Cambria',...
    'Fontsize', 9);

% There are four list boxes that quantify what a peak looks like....

h.peak_panel = uibuttongroup(...
    'Parent', h.view_panel,...
    'Units', 'normalized',...
    'Position', [0.05 0.61 0.90 0.38],...
    'FontName', 'Cambria',...
    'Fontsize', 9,...
    'Title', 'Peak Parameters');

h.rise_textbox = uicontrol( ...
    'Parent', h.peak_panel,...
    'Style', 'text', ...
    'Units', 'normalized', ... 
    'Position', [0.05 0.81 0.65 0.17], ...
    'HorizontalAlignment', 'Right',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'String', 'required rise (%):');

h.rise_listbox = uicontrol(...
    'Parent', h.peak_panel,...
    'Style', 'popupmenu',...
    'Units', 'normalized',...
    'Position', [0.75 0.81 0.20 0.19],...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontWeight', 'bold',...
    'BackgroundColor', [0.5 0.5 0.5],...
    'ForegroundColor', [1 1 1],...
    'HorizontalAlignment', 'Left',...
    'String', 0:100,...
    'Value', 16);

%

h.before_textbox = uicontrol( ...
    'Parent', h.peak_panel,...
    'Style', 'text', ...
    'Units', 'normalized', ... 
    'Position', [0.05 0.58 0.65 0.17], ...
    'HorizontalAlignment', 'Right',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'String', 'max lookback (pts):');

h.before_listbox = uicontrol(...
    'Parent', h.peak_panel,...
    'Style', 'popupmenu',...
    'Units', 'normalized',...
    'Position', [0.75 0.58 0.20 0.19],...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontWeight', 'bold',...
    'BackgroundColor', [0.5 0.5 0.5],...
    'ForegroundColor', [1 1 1],...
    'HorizontalAlignment', 'Left',...
    'String', 0:100,...
    'Value', 26);

%

h.fall_textbox = uicontrol( ...
    'Parent', h.peak_panel,...
    'Style', 'text', ...
    'Units', 'normalized', ...
    'Position', [0.05 0.35 0.65 0.17], ...
    'HorizontalAlignment', 'Right',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'String', 'required fall (%):');

h.fall_listbox = uicontrol(...
    'Parent', h.peak_panel,...
    'Style', 'popupmenu',...
    'Units', 'normalized',...
    'Position', [0.75 0.35 0.20 0.19],...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontWeight', 'bold',...
    'BackgroundColor', [0.5 0.5 0.5],...
    'ForegroundColor', [1 1 1],...
    'HorizontalAlignment', 'Left',...
    'String', 0:100,...
    'Value', 16);

%

h.after_textbox = uicontrol( ...
    'Parent', h.peak_panel,...
    'Style', 'text', ...
    'Units', 'normalized', ...
    'Position', [0.05 0.12 0.65 0.17], ...
    'HorizontalAlignment', 'Right',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'String', 'max lookahead (pts):');

h.after_listbox = uicontrol(...
    'Parent', h.peak_panel,...
    'Style', 'popupmenu',...
    'Units', 'normalized',...
    'Position', [0.75 0.12 0.20 0.19],...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontWeight', 'bold',...
    'BackgroundColor', [0.5 0.5 0.5],...
    'ForegroundColor', [1 1 1],...
    'HorizontalAlignment', 'Left',...
    'String', 0:100,...
    'Value', 26);

% Sometimes the data doesn't move much -- and it might be nice to detect
% jumps that are big RELATIVE to the size of the adjusted range, instead of
% defining everything ABSOLUTELY....

h.absolute_button = uicontrol(...
    'Parent', h.peak_panel,...
    'Style', 'radiobutton',...
    'Units', 'normalized',...
    'Value', 0,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontAngle', 'italic',...
    'Position', [0.20 0.01 0.30 0.10],...
    'String', 'absolute %');

h.relative_button = uicontrol(...
    'Parent', h.peak_panel,...
    'Style', 'radiobutton',...
    'Units', 'normalized',...
    'Value', 0,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontAngle', 'italic',...
    'Position', [0.50 0.01 0.40 0.10],...
    'String', 'relative to range');

% The user might like to see how the newly defined peaks compare to those
% of our old PeakFinder software -- or to fix the occasional data point
% that takes a picture of the lens cap.... 

h.option_panel = uipanel(...
    'Parent', h.view_panel,...
    'Units', 'normalized',...
    'Position', [0.05 0.42 0.90 0.16],...
    'FontName', 'Cambria',...
    'Fontsize', 9,...
    'Title', 'Options');

h.shutter_checkbox = uicontrol(...
    'Style', 'checkbox',...
    'Parent', h.option_panel,...
    'Units', 'normalized',...
    'String', 'Interpolate across closed shutters?',...
    'Value', 1,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'Position', [0.05 0.52 0.90 0.42]);

h.old_checkbox = uicontrol(...
    'Style', 'checkbox',...
    'Parent', h.option_panel,...
    'Units', 'normalized',...
    'String', 'Show "old" PeakFinder peaks?',...
    'Value', 1,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'Position', [0.05 0.05 0.90 0.42]);

% We have options to make the crooked straight and the rough places
% plain.... 

h.trend_panel = uibuttongroup(...
    'Parent', h.view_panel,...
    'Units', 'normalized',...
    'Position', [0.05 0.08 0.90 0.32],...
    'FontName', 'Cambria',...
    'Fontsize', 9,...
    'Title', 'Trend Control');

h.no_trend_button = uicontrol(...
    'Style', 'radiobutton',...
    'Units', 'normalized',...
    'Parent', h.trend_panel,...
    'String', 'no trend',...
    'Value', 0,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'Position', [0.05 0.81 0.9 0.16]);

h.convex_button = uicontrol(...
    'Style', 'radiobutton',...
    'Units', 'normalized',...
    'Parent', h.trend_panel,...
    'String', 'Convex Envelope',...
    'Value', 0,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'Position', [0.05 0.62 0.9 0.16]);

h.one_sided_ema_button = uicontrol(...
    'Style', 'radiobutton',...
    'Units', 'normalized',...
    'Parent', h.trend_panel,...
    'String', 'Exponential Moving Average (1-sided)',...
    'Value', 0,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'Position', [0.05 0.43 0.9 0.16]);

h.two_sided_ema_button = uicontrol(...
    'Style', 'radiobutton',...
    'Units', 'normalized',...
    'Parent', h.trend_panel,...
    'String', 'Exponential Moving Average (2-sided)',...
    'Value', 1,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'Position', [0.05 0.24 0.9 0.16]);

h.diffusion_button = uicontrol(...
    'Style', 'radiobutton',...
    'Units', 'normalized',...
    'Parent', h.trend_panel,...
    'String', 'Finite Difference Diffusion (2-sided)',...
    'Value', 1,...
    'HorizontalAlignment', 'center',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'Position', [0.05 0.05 0.9 0.16]);

h.smoothness_textbox = uicontrol( ...
    'Parent', h.view_panel,...
    'Style', 'text', ...
    'Units', 'normalized', ... 
    'Position', [0.05 0.01 0.71 0.06], ...
    'HorizontalAlignment', 'Right',...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontAngle', 'italic',...
    'String', 'trend smoothness');

h.smoothness_listbox = uicontrol(...
    'Parent', h.view_panel,...
    'Style', 'popupmenu',...
    'Units', 'normalized',...
    'Position', [0.78 0.01 0.17 0.07],...
    'FontName', 'Calibri',...
    'Fontsize', 9,...
    'FontWeight', 'bold',...
    'FontAngle', 'italic',...
    'BackgroundColor', [0.9 0.9 0.9],...
    'HorizontalAlignment', 'Left',...
    'String', 1:200,...
    'Value', 60);

% Once the parameters are set, RUN!!!

h.run_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 16,...
    'BackgroundColor', [0.3 0.2 0.1],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.71 0.18 0.24 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'Find Peaks');

% Once the run is done, offer to pop out the resulting graph....

h.graph_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 12,...
    'BackgroundColor', [0.7 0.3 0.3],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.76 0.12 0.07 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'Pop Graph');

% Or simultaneously view all of the fluorescence traces....

h.waterfall_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 12,...
    'BackgroundColor', [0.7 0.5 0.3],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.845 0.12 0.07 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'View Traces');

% Or see a raster plot of the peaks....

h.raster_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 12,...
    'BackgroundColor', [0.7 0.7 0.3],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.88 0.07 0.07 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'Raster Plot');

% Or see a histogram of the rise and fall times....

h.histogram_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 12,...
    'BackgroundColor', [0.4 0.3 0.6],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.71 0.07 0.07 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'Histograms');

% Or check out the cross-correlations....

h.cross_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 12,...
    'BackgroundColor', [0.3 0.3 0.7],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.76 0.02 0.07 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'Correlations');

% Or see the synchronization matrix....

h.synchronization_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 12,...
    'BackgroundColor', [0.4 0.6 0.4],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.845 0.02 0.07 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'Synchronicity');

% Or save the data about peaks to a spreadsheet....

h.output_button = uicontrol(...
    'Style', 'pushbutton',...
    'Units', 'normalized',...
    'FontName', 'Calibri',...
    'Fontsize', 12,...
    'BackgroundColor', [0.3 0.3 0.3],...
    'ForegroundColor', [1 1 1],...
    'Position', [0.795 0.07 0.07 0.04],...
    'Interruptible', 'off',...
    'Visible', 'off',...
    'String', 'Save Output');

% We have several tables (not displayed) for storing the data following a
% run.... 

h.time_table = uitable(...
    'Units', 'normalized',...
    'Position', [0 0 1 1 ],...
    'Visible' , 'off');

h.observation_table = uitable(...
    'Units', 'normalized',...
    'Position', [0 0 1 1 ],...
    'Visible' , 'off');

h.shutter_free_table = uitable(...
    'Units', 'normalized',...
    'Position', [0 0 1 1 ],...
    'Visible' , 'off');

h.peak_table = uitable(...
    'Units', 'normalized',...
    'Position', [0 0 1 1 ],...
    'Visible' , 'off');

h.before_and_after_table = uitable(...
    'Units', 'normalized',...
    'Position', [0 0 1 1 ],...
    'Visible' , 'off');

h.half_table = uitable(...
    'Units', 'normalized',...
    'Position', [0 0 1 1 ],...
    'Visible' , 'off');

% With the figure elements in place, we define the callbacks. 
% When the user makes a choice, we respond accordingly.... 

set(h.file_button , 'callback', {@load_data_callback, h})
set(h.run_button , 'callback', {@run_peakcaller_callback, h})
set(h.series_slider , 'callback', {@slider_callback, h})

set(h.graph_button , 'callback', {@pop_graph_callback, h})
set(h.cross_button , 'callback', {@view_correlations_callback, h})
set(h.output_button , 'callback', {@save_output_callback, h})

set(h.raster_button , 'callback', {@view_raster_callback, h})
set(h.histogram_button , 'callback', {@view_histogram_callback, h})
set(h.waterfall_button , 'callback', {@view_waterfall_callback, h})
set(h.synchronization_button , 'callback', {@view_synchronization_callback, h})
end

%%

function load_data_callback(~,~,h)

% The user should choose a file to analyze... we let them choose!

[FileName, FilePath, FlagUp]=uigetfile({'*.txt'; '*.csv'; '*.xls'; '*.xlsx'; '*.*'}, 'Choose Tab-Delimited Data File');

% If the user didn't click "cancel", we try to open the file.

if FlagUp
    try
        % The data for a Spinning Disk file starts in the third row. The
        % first column is time stamps -- and dlmread tends to read in a
        % final column of zeros which we discard.
        
        RawData = dlmread([FilePath FileName],'',2,0);
        TimeStamps = RawData(:,1);
        Observations = RawData(:,2:end-1);
        set(h.file_textbox, 'String', FileName)
        set(h.type_textbox, 'String', '(Spinning Disk)')
        
        set(h.view_panel, 'Visible', 'on')
        set(h.run_button, 'Visible', 'on')
        
        cla(h.original_axes, 'reset')
        cla(h.adjusted_axes, 'reset')
        set(h.series_slider, 'Visible', 'off')
        set(h.series_text, 'Visible', 'off')
        
        set(h.graph_button, 'Visible', 'off')
        set(h.output_button, 'Visible', 'off')
        set(h.cross_button, 'Visible', 'off')
        set(h.histogram_button, 'Visible', 'off')
        set(h.waterfall_button, 'Visible', 'off')
        set(h.raster_button, 'Visible', 'off')
        set(h.synchronization_button, 'Visible', 'off')
        
    catch 
        try
            % If the format is wrong for a Spinning Disk file, we try to
            % read it as an Array Scan file. In an Array Scan file, the
            % data we want starts in the second row and the fourth column.
            % Time is not provided, and the data is transposed....

            RawData = dlmread([FilePath FileName],'\t',1,3);
            TimeStamps = [];
            Observations = RawData';
            GoodSeries = (sum(Observations>0)==max(sum(Observations>0)));
            Observations = Observations(1:max(sum(Observations>0)),GoodSeries);
            set(h.file_textbox, 'String', FileName)
            set(h.type_textbox, 'String', '(Array Scan)')
            
            set(h.view_panel, 'Visible', 'on')
            set(h.run_button, 'Visible', 'on')
            
            cla(h.original_axes, 'reset')
            cla(h.adjusted_axes, 'reset')
            set(h.series_slider, 'Visible', 'off')
            set(h.series_text, 'Visible', 'off')
            
            set(h.graph_button, 'Visible', 'off')
            set(h.output_button, 'Visible', 'off')
            set(h.cross_button, 'Visible', 'off')
            set(h.histogram_button, 'Visible', 'off')
            set(h.waterfall_button, 'Visible', 'off')
            set(h.raster_button, 'Visible', 'off')
            set(h.synchronization_button, 'Visible', 'off')
        catch 
            % If neither format works, tell the user....
            
            errordlg('This is not the file you''re looking for.... Go about your business.... Move along....', 'Neither Array Scan NOR Spinning Disk format.')
            return
        end
    end
    
    % Store the data and set up the slider so that we can scroll through
    % all N series using the scroll bar....
    
    set(h.time_table, 'Data', TimeStamps);
    set(h.observation_table, 'Data', Observations);
    
    [T,N] = size(Observations);
    
    set(h.series_slider, 'Value', 1)
    set(h.series_slider, 'Max', N)
    set(h.series_slider, 'SliderStep', [1/(N-1) 1/(N-1)])
    
    set(h.series_text, 'String', 1)
    
    % Look for closed shutters. We record a series in a duplicate table
    % with any of these defects removed....
    
    for ii = 1:N
        if Observations(1,ii) < 0.01*Observations(2,ii)
            Observations(1,ii) = Observations(2,ii);
        end
        for jj = 2:T-1
            if Observations(jj,ii) < 0.01*Observations(jj-1,ii)
                if Observations(jj,ii) < 0.01*Observations(jj+1,ii)
                    Observations(jj,ii) = 0.5*(Observations(jj-1,ii) + Observations(jj+1,ii));
                end
            end
        end
        if Observations(T,ii) < 0.01*Observations(T-1,ii)
            Observations(T,ii) = Observations(T-1,ii);
        end
    end
    
    set(h.shutter_free_table, 'Data', Observations);

end

end

%%

function run_peakcaller_callback(~,~,h)

% This is the heart of PeakCaller! Here we take note of the user's 
% requests and compute the peaks in the chosen data....

% First, we retrieve the data -- with or without corrected shutter shots,
% accoring to the user's desires....

if get(h.shutter_checkbox,'Value')
    Observations = get(h.shutter_free_table, 'Data');
else
    Observations = get(h.observation_table, 'Data');
end

[T,N] = size(Observations);
    
% We determine the trend in the data using the user's preferred method....

TrendSmoothness = get(h.smoothness_listbox, 'Value');

SmoothedObservations = zeros(T,N);

for ii=1:N
    if (ii==1) && ( get(h.one_sided_ema_button, 'Value') || get(h.two_sided_ema_button, 'Value') )
        % This is the code for a traditional exponential moving average.
        MatrixSmoother = zeros(T);
        p=2/(TrendSmoothness+1);
        q = 1-p;
        for jj = 0:T-1
            MatrixSmoother(2:T,2:T) = MatrixSmoother(2:T,2:T)+q^jj*p*diag(ones(T-1-jj,1),-jj);
            MatrixSmoother(jj+1,1) = q^jj;
        end
        if get(h.two_sided_ema_button, 'Value')
            % For a two-sided EMA, we average the two one-sided exponential
            % moving averages coming from each end....
            MatrixSmoother = (MatrixSmoother + rot90(MatrixSmoother,2))/2;
        end
    elseif (ii==1) && get(h.diffusion_button, 'Value')
        % This is just a numerical solution of the heat or diffusion
        % equation with insulated ends....
        FiniteDifference = diag(-2*ones(T,1))+diag(ones(T-1,1),1)+diag(ones(T-1,1),-1);
        FiniteDifference(1,2) = 2;
        FiniteDifference(T,T-1) = 2;
        MatrixSmoother = (eye(T)+FiniteDifference/4)^(4*TrendSmoothness);
    elseif (ii==1) && get(h.no_trend_button, 'Value')
        % For no trend, we just take the average value all the way
        % across....
        MatrixSmoother = ones(T)/T;
    elseif get(h.convex_button, 'Value')
        % A nice geometric trend, we simply connect the dots BELOW the
        % signal. Note that whereas the previous matrices are independent
        % of the signal, this one changes with each new signal....
        MatrixSmoother = zeros(T);
        PriorIndex = T;
        while PriorIndex > 1
            SecantSlope = (Observations(PriorIndex,ii) - Observations(1:PriorIndex-1,ii))...
                ./ (1-PriorIndex:-1)';
            [~,NewIndex] = min(SecantSlope);
            MatrixSmoother(NewIndex:PriorIndex,NewIndex)=(PriorIndex-NewIndex:-1:0)/(PriorIndex-NewIndex);
            MatrixSmoother(NewIndex:PriorIndex,PriorIndex)=(0:PriorIndex-NewIndex)/(PriorIndex-NewIndex);
            PriorIndex = NewIndex;
        end
    end
    
    SmoothedObservations(:,ii) = MatrixSmoother*Observations(:,ii);
end

% Save the smoothed observations....

set(h.observation_table, 'UserData', SmoothedObservations);

% Note that all computations are done with the DETRENDED data....

DetrendedObservations = Observations./SmoothedObservations;

% Old fashioned local maxima...

% A LOCAL MAX is any value that is bigger than the single data point
% immediately before it and the single data point immediately after it. 
% These local maxima are candidates for peaks.

Candidate = false(T,N);

for ii = 1:N
    Candidate(:,ii) = ([-inf;DetrendedObservations(1:end-1,ii)] < DetrendedObservations(:,ii)) & ...
        (DetrendedObservations(:,ii) > [DetrendedObservations(2:end,ii); -inf]);
end

% A PEAK is a very special sort of local max. It needs to be 10% bigger
% (adjustable) than nearby lows before and after. 

% Before a peak, we require values 10% (or so) lower than the candidate:
%     (1) in the previous second (adjustable) -- or the beginning of the 
%         data set if we're in the first second,
% OR  (2) after the most recent peak,
% whichever is most recent....

% After a peak, we require values 10% (or so) lower than the candidate:
%    (1) in the following second (separately adjustable) -- or the end 
%        of the data set if we're in the final second,
% OR (2) before values become larger than the current candidate,
% whichever happens first....  

Peaks = false(T,N);
Before = false(T,N);
After = false(T,N);
HalfBefore = false(T,N);
HalfAfter = false(T,N);

RequestedRise = (get(h.rise_listbox, 'Value')-1)/100;
RequestedFall = (get(h.fall_listbox, 'Value')-1)/100;
TrendSmoothness = get(h.before_listbox, 'Value')-1;
LookAhead = get(h.after_listbox, 'Value')-1;

RelativeRequest = get(h.relative_button, 'Value');

for ii = 1:N
    if RelativeRequest
        RequiredRise = RequestedRise*(max(DetrendedObservations(:,ii))-min(DetrendedObservations(:,ii)));
        RequiredFall = RequestedFall*(max(DetrendedObservations(:,ii))-min(DetrendedObservations(:,ii)));
    else
        RequiredRise = RequestedRise;
        RequiredFall = RequestedFall;
    end
    PriorPeak = 1;
    % Forward Pass
    for jj=2:T-1
        if Candidate(jj,ii)
            LookBackIndex = max([PriorPeak, jj-TrendSmoothness]);
            [MinBefore,IndexBefore] = min(DetrendedObservations(LookBackIndex:jj-1,ii));
            if MinBefore < (1-RequiredRise)*DetrendedObservations(jj,ii)
                TooTall = (DetrendedObservations(jj+1:min(jj+LookAhead,T),ii) > DetrendedObservations(jj,ii));
                TooTallIndex = find(TooTall,1);
                if isempty(TooTallIndex)
                    LookAheadIndex = min(jj+LookAhead,T);
                else
                    LookAheadIndex = jj+TooTallIndex;
                end
                MinAfter = min(DetrendedObservations(jj+1:LookAheadIndex,ii));
                if MinAfter < (1-RequiredFall)*DetrendedObservations(jj,ii)
                    Peaks(jj,ii) = true;
                    Before(LookBackIndex+IndexBefore-1, ii) = true;
                    PriorPeak = jj;
                    HalfIndex = find(DetrendedObservations(LookBackIndex:jj-1,ii)<0.5*(MinBefore+DetrendedObservations(jj,ii)),1, 'last');
                    HalfBefore(LookBackIndex+HalfIndex-1, ii) = true;
                end
            end
        end
    end
    % Backward Pass -- to better identify MinAfter, and tweak peaks
    % in the case when RequiredRise and RequiredFall are different.
    NextPeak = T;
    for jj=T-1:-1:2
        if Peaks(jj,ii)
            LookForwardIndex = min([NextPeak, jj+TrendSmoothness]);
            [MinAfter,IndexAfter] = min(DetrendedObservations(jj+1:LookForwardIndex,ii));
            if MinAfter < (1-RequiredFall)*DetrendedObservations(jj,ii)
                % If there's a big enough drop between one peak and the 
                % next, the peak is confirmed!
                After(jj+IndexAfter, ii) = true;
                NextPeak = jj;
                HalfIndex = find(DetrendedObservations(jj+1:jj+IndexAfter,ii)<0.5*(MinAfter+DetrendedObservations(jj,ii)),1, 'first');
                HalfAfter(jj+HalfIndex, ii) = true;
            else
                % If there's not, the peak is deleted!
                Peaks(jj,ii) = false;
                IndexBefore = find(Before(1:jj,ii),1, 'last');
                Before(IndexBefore,ii) = false;
                HalfIndex = find(HalfBefore(1:jj,ii),1, 'last');
                HalfBefore(HalfIndex,ii) = false;
            end
        end
    end
end

% Save the peaks....

set(h.peak_table, 'Data', Peaks)

set(h.before_and_after_table, 'Data', Before)
set(h.before_and_after_table, 'UserData', After)

set(h.half_table, 'Data', HalfBefore)
set(h.half_table, 'UserData', HalfAfter)

% If the user wants to see the "old" peaks, we compute them as well....

OldPeaks = false(T,N);

if get(h.old_checkbox', 'Value')
    for ii=1:N
        [~,PeakIndices,~,ValleyIndices] = old_extrema(Observations(:,ii));
        if ~isempty(PeakIndices)
            indices = old_relevantPeaks2(sort(PeakIndices)',sort(ValleyIndices)',Observations(:,ii),1:T);
            OldPeaks(indices,ii) = true;
        end
    end
end

set(h.peak_table, 'UserData', OldPeaks)

% Show the buttons so the user can interpret the results....

set(h.series_slider, 'Visible', 'on')
set(h.series_text, 'Visible', 'on')

set(h.graph_button, 'Visible', 'on')
set(h.output_button, 'Visible', 'on')
set(h.cross_button, 'Visible', 'on')
set(h.histogram_button, 'Visible', 'on')
set(h.waterfall_button, 'Visible', 'on')
set(h.raster_button, 'Visible', 'on')
set(h.synchronization_button, 'Visible', 'on')

PlotSeries(h)
end

%%

function PlotSeries(h)

% Get all of the appropriate data....

if get(h.shutter_checkbox,'Value')
    Observations = get(h.shutter_free_table, 'Data');
else
    Observations = get(h.observation_table, 'Data');
end

SmoothedObservations = get(h.observation_table, 'UserData');
Peaks = get(h.peak_table, 'Data');
OldPeaks = get(h.peak_table, 'UserData');

% Find out which series to plot....

SeriesNumber = get(h.series_slider, 'Value');

T = size(Observations,1);
Indices = 1:T;

% On the top axes, plot the Observed Signal with the smoothed trend
% underneath. Both the peaks and old peaks (if they've been computed) are
% shown....

plot(h.original_axes,SmoothedObservations(:,SeriesNumber),'Color',[1 0.6 0.6],'LineWidth',2)
hold(h.original_axes,'on')
plot(h.original_axes,Observations(:,SeriesNumber),'Color',[0 0 0],'LineWidth',1)
plot(h.original_axes,Indices(Peaks(:,SeriesNumber)),Observations(Peaks(:,SeriesNumber),SeriesNumber),'go')
plot(h.original_axes,Indices(OldPeaks(:,SeriesNumber)),Observations(OldPeaks(:,SeriesNumber),SeriesNumber),'rx')
xlim(h.original_axes,[1 T])
hold(h.original_axes,'off')

% On the bottom axes, plot the Detrended Signal. Again, both the peaks and
% old peaks (if they've been computed) are shown....

plot(h.adjusted_axes,Observations(:,SeriesNumber)./SmoothedObservations(:,SeriesNumber),'Color',[0 0 0],'LineWidth',1)
xlim(h.adjusted_axes,[1 T])
hold(h.adjusted_axes,'on')
plot(h.adjusted_axes,Indices(Peaks(:,SeriesNumber)),Observations(Peaks(:,SeriesNumber),SeriesNumber)./SmoothedObservations(Peaks(:,SeriesNumber),SeriesNumber),'go')
plot(h.adjusted_axes,Indices(OldPeaks(:,SeriesNumber)),Observations(OldPeaks(:,SeriesNumber),SeriesNumber)./SmoothedObservations(OldPeaks(:,SeriesNumber),SeriesNumber),'rx')

hold(h.adjusted_axes,'off')

end

%%

function pop_graph_callback(~,~,h)

% Create a new figure for the visible trace, with all the menu bars for
% easy manipulation.... 

figure('Name', 'Printable Figure Window', 'Color', [0.9 0.9 0.9]);

% Get all of the appropriate data....

if get(h.shutter_checkbox,'Value')
    Observations = get(h.shutter_free_table, 'Data');
else
    Observations = get(h.observation_table, 'Data');
end

SmoothedObservations = get(h.observation_table, 'UserData');
Peaks = get(h.peak_table, 'Data');

T = size(Observations,1);
Time = get(h.time_table, 'Data');
if length(Time) ~= T
    Time = 1:T;
end

% Find out which series to plot....

SeriesNumber = get(h.series_slider, 'Value');

% Plot the Detrended Signal with the peaks shown....

plot(Time, Observations(:,SeriesNumber)./SmoothedObservations(:,SeriesNumber),'Color',[0 0 0],'LineWidth',1)
hold('on')
plot(Time(Peaks(:,SeriesNumber)),Observations(Peaks(:,SeriesNumber),SeriesNumber)./SmoothedObservations(Peaks(:,SeriesNumber),SeriesNumber),'go')
xlim([Time(1) Time(T)])

xlabel('Time (s)')
ylabel('Intensity')
title(['Fluorescence Trace ', num2str(SeriesNumber)])

hold(h.adjusted_axes,'off')

end

%%

function view_raster_callback(~,~,h)

% This function allows us to get a visual picture of the peaks across the
% different ROIs in the file....

Peaks = get(h.peak_table, 'Data');
Before = get(h.before_and_after_table, 'Data');
After = get(h.before_and_after_table, 'UserData');

HalfBefore = get(h.half_table, 'Data');
HalfAfter = get(h.half_table, 'UserData');

[T,N] = size(Peaks);

Time = get(h.time_table, 'Data');
if length(Time) ~= T
    Time = 1:T;
end

[PeakLocation,PeakIndex] = find(Peaks);
[BeforeLocation,BeforeIndex] = find(Before);
[AfterLocation,AfterIndex] = find(After);
[HalfBeforeLocation,HalfBeforeIndex] = find(HalfBefore);
[HalfAfterLocation,HalfAfterIndex] = find(HalfAfter);

figure('Name', 'Raster Plot', 'Color', [0.9 0.9 0.9]);

% For each peak, we plot a thin line from the preceding minimum to the one
% after, and a thicker line from the point halfway up to the peak until 
% it's halfway back down....

for ii=1:size(PeakLocation)
    plot([Time(BeforeLocation(ii)),Time(AfterLocation(ii))], [BeforeIndex(ii),AfterIndex(ii)],'o-',...
        'Color',[0.8 1 0.8],...
        'LineWidth',1,...
        'MarkerSize',3,...
        'MarkerEdgeColor',[0.7 1 0.7])
    hold on
    plot([Time(HalfBeforeLocation(ii)),Time(HalfAfterLocation(ii))], [HalfBeforeIndex(ii),HalfAfterIndex(ii)],'-',...
        'Color',[0.6 1 0.6],...
        'LineWidth',4)
end

% And we put a nice big filled circle on each peak....

plot(Time(PeakLocation), PeakIndex, 'o',...
        'MarkerSize',6,...
        'MarkerEdgeColor','[0 0.8 0]',...
        'MarkerFaceColor','[0 0.8 0]')
hold off

xlim([0,Time(T)+Time(2)-Time(1)])
ylim([0,N+1])
xlabel('Time (s)')
ylabel('ROI (#)')
title('Called Peaks')
end

%%

function view_synchronization_callback(~,~,h)

% Compute a synchronization index, based on our identified peaks....

HalfBefore = get(h.half_table, 'Data')';
HalfAfter = get(h.half_table, 'UserData')';

[N,T] = size(HalfBefore);

% We generalize our fluorescence traces to highlight just the peaks. In
% this generalized function, near a peak is a +1, and not near a peak is a
% -1....  

PeakRegions = zeros(N,T);
for ii=1:N
    PeakRegions(ii,1) = HalfBefore(ii,1) - HalfAfter(ii,1);
    for jj=2:T
    PeakRegions(ii,jj) = PeakRegions(ii,jj-1) + HalfBefore(ii,jj) - HalfAfter(ii,jj);
    end
end
PeakRegions = 2*PeakRegions - 1;

% We build a generalized autocorrelation function (P) based upon our
% generalized peaks....

P = zeros(N,T-1);
for ii=1:N
    R = PeakRegions(ii,:)'*PeakRegions(ii,:);
    for jj=1:T-1
        P(ii,jj) = mean(R(jj*T+1:T+1:end));
    end
end

WaitHandle = waitbar(0,'Computing Synchronization Indices...');

% Finally, we compute the pairwise correlation of the generalized
% autocorrelation functions -- this yields our synchronization matrix.

% Note that the matrix SI is initiated to be one row and one column too
% large. This is because MATLAB's function pcolor ignores the last row and
% column in faceted shading, and we don't want to ignore actual data....

SI = zeros(N+1);
for ii=1:N
    for jj=1:ii
        SI(ii,jj) = sum((P(ii,:)-mean(P(ii,:))).*(P(jj,:)-mean(P(jj,:)))./std(P(ii,:))./std(P(jj,:)))/(T-2);
        SI(jj,ii) = SI(ii,jj);
    end
    waitbar(ii/N);
end

% Note that a trace that does not include any peaks will be a constant
% function -- and thus will result in a division by zero above. We use the
% convention that the correlation of any function with a constant is 0.

SI(isnan(SI)) = 0;
SImean = nansum(nansum(SI - diag(diag(SI))))/(N*(N-1));

delete(WaitHandle)

figure('Name', 'Synchronization Matrix', 'Color', [0.9 0.9 0.9]);

RawFigurePosition = get(gcf,'position');

pcolor(SI)
colormap(jet)
title(sprintf('Mean Synchronization Index: %1.3f', SImean));
colorbar;
RawMatrix = gca;
RawMatrix.YTick = (1:N) + 0.5;
RawMatrix.XTick = (1:N) + 0.5;
RawMatrix.YTickLabel = 1:N;
RawMatrix.XTickLabel = 1:N;

% Pop out a second figure that shows the clusters....

[evectors, evalues] = eig(SI(1:end-1,1:end-1));
evalues = diag(evalues);
Bigevalues = evalues(evalues > 1);
Bigevectors = evectors(:,evalues > 1);

MaxScore = zeros(N,1);
MaxIndex = MaxScore;

if ~isempty(Bigevalues)
    ParticipationIndices = (Bigevectors.*abs(Bigevectors))*diag(Bigevalues);
    for ii=1:N
        [~, MaxIndex(ii)] = max(abs(ParticipationIndices(ii,:)));
        MaxScore(ii) = ParticipationIndices(ii,MaxIndex(ii));
    end
end

MaxIndex(abs(MaxScore) < eps) = 0;
ScoreMatrix = [MaxScore, MaxIndex];

[~,ReArrangement] = sortrows(ScoreMatrix,[-2 -1]);

figure('Name', 'Synchronization Matrix (Rearranged)',...
    'Color', [0.9 0.9 0.9],...
    'Position', RawFigurePosition + [50 -50 0 0]);

pcolor(SI([ReArrangement; N+1],[ReArrangement; N+1]))
colormap(jet)
title({sprintf('Mean Synchronization Index: %1.3f', SImean);...
    sprintf('%d cluster(s) identified', length(Bigevalues))});
colorbar;
SortedMatrix = gca;
SortedMatrix.YTick = (1:N) + 0.5;
SortedMatrix.XTick = (1:N) + 0.5;
SortedMatrix.YTickLabel = ReArrangement;
SortedMatrix.XTickLabel = ReArrangement;

hold on
StartBox=1;
for ii = length(Bigevalues):-1:1
    EndBox = sum(MaxIndex>=ii)+1;
    plot([StartBox EndBox EndBox StartBox StartBox],[StartBox StartBox EndBox EndBox StartBox],'color',[0.9 0.9 0.9],'linewidth',2)
    StartBox = EndBox;
end
plot([1 N+1 N+1 1 1 ],[1 1 N+1 N+1 1],'color',[0.9 0.9 0.9],'linewidth',2)
end

%%

function view_waterfall_callback(~,~,h)

% The fluorescence traces are plotted, with peaks highlighted. 

% Go get all of the data that we wish to view....

if get(h.shutter_checkbox,'Value')
    Observations = get(h.shutter_free_table, 'Data')';
else
    Observations = get(h.observation_table, 'Data')';
end

SmoothedObservations = get(h.observation_table, 'UserData')';
Observations = Observations./SmoothedObservations;

HalfBefore = get(h.half_table, 'Data')';
HalfAfter = get(h.half_table, 'UserData')';

[N,T] = size(HalfBefore);

% Choose a color scheme to highlight the peaks....

ColorChart = zeros(N,T);

for ii=1:N
    ColorChart(ii,1) = -HalfBefore(ii,1) + HalfAfter(ii,1);
    for jj=2:T
    ColorChart(ii,jj) = ColorChart(ii,jj-1) - HalfBefore(ii,jj) + HalfAfter(ii,jj);
    end
end

% Plot it as a waterfall, and choose a nice angle from which to view it....

figure('Name', 'Fluorescence Traces', 'Color', [0.9 0.9 0.9]);

waterfall(Observations,ColorChart)
colormap(copper)
xlim([0,T+1])
ylim([0,N+1])
view([-1,-1,8])
end

%%

function view_correlations_callback(~,~,h)

% We look for cross-correlations between the different fluorescence traces,
% and plot a heat map showing the max correlation for lags between -max and
% max....

if get(h.shutter_checkbox,'Value')
    Observations = get(h.shutter_free_table, 'Data');
else
    Observations = get(h.observation_table, 'Data');
end

SmoothedObservations = get(h.observation_table, 'UserData');
Observations = Observations./SmoothedObservations;

[T,N] = size(Observations);

% When comparing traces, don't slide over more than 50 ticks (or half the
% window if that's smaller)....

MaxLag = min(50,T/2);

% Note that HeatMatrix is initiated to be one row and one column too large.
% This is because MATLAB's function pcolor ignores the last row and column
% in faceted shading, and we don't want to ignore actual data.... 

HeatMatrix = zeros(N+1);

WaitHandle = waitbar(0,'Computing Correlations...');

% Slide forward and backward, and save the best fit....

for ii=1:N
    for jj=1:ii
        MaxCorrelation = -1;
        for Lag = 0:MaxLag
            A = Observations(1:T-Lag,ii);
            B = Observations(1+Lag:T,jj);
            LaggedCorrelation = sum((A-mean(A)).*(B-mean(B))./std(A)./std(B))/(T-Lag-1);
            MaxCorrelation = max(MaxCorrelation,LaggedCorrelation);
        end
        for Lag = 1:MaxLag
            A = Observations(1+Lag:T,ii);
            B = Observations(1:T-Lag,jj);
            LaggedCorrelation = sum((A-mean(A)).*(B-mean(B))./std(A)./std(B))/(T-Lag-1);
            MaxCorrelation = max(MaxCorrelation,LaggedCorrelation);
        end
        HeatMatrix(ii,jj) = MaxCorrelation;
        HeatMatrix(jj,ii) = MaxCorrelation;
    end
    waitbar(ii/N);
end

delete(WaitHandle)

figure('Name', 'Cross-Correlation Matrix', 'Color', [0.9 0.9 0.9]);
pcolor(HeatMatrix)
colormap(jet)
colorbar;
AxisHandle = gca;
AxisHandle.YTick = (1:N) + 0.5;
AxisHandle.XTick = (1:N) + 0.5;
AxisHandle.YTickLabel = 1:N;
AxisHandle.XTickLabel = 1:N;

end

%%

function view_histogram_callback(~,~,h)

% Two histograms are plotted, showing the distribution of (1) peak heights
% and (2) rise time and fall time

% Go get all of the data that we wish to summarize and save....

if get(h.shutter_checkbox,'Value')
    Observations = get(h.shutter_free_table, 'Data');
else
    Observations = get(h.observation_table, 'Data');
end

SmoothedObservations = get(h.observation_table, 'UserData');
Peaks = get(h.peak_table, 'Data');
Time = get(h.time_table, 'Data');

HalfBefore = get(h.half_table, 'Data');
HalfAfter = get(h.half_table, 'UserData');

T = size(Peaks,1);

% If there's no time series provided, we improvise....

if length(Time) ~= T
    Time = 1:T;
end

% Pull out the peaks and the valleys....

[PeakLocation,PeakIndex] = find(Peaks);
[HalfBeforeLocation,~] = find(HalfBefore);
[HalfAfterLocation,~] = find(HalfAfter);
NumPeaks = length(PeakLocation);

% For each peak, make a note of it's height and the rise and fall times....

PeakHeight = zeros(NumPeaks,1);
RiseTime = PeakHeight;
FallTime = PeakHeight;

for ii=1:NumPeaks
        PeakHeight(ii) = Observations(PeakLocation(ii),PeakIndex(ii))./SmoothedObservations(PeakLocation(ii),PeakIndex(ii)) - 1;
        RiseTime(ii) = Time(PeakLocation(ii))-Time(HalfBeforeLocation(ii));
        FallTime(ii) = Time(HalfAfterLocation(ii))-Time(PeakLocation(ii));
end

figure('Name', 'Peak Property Histograms', 'Color', [0.9 0.9 0.9]);

% On the left-hand plot, make a histogram of the peak heights....

subplot(1,2,1)
histogram(PeakHeight,10,'FaceColor',[0.4,0.4,0.6],'EdgeColor',[0.2,0.2,0.3])
xlabel('height of peak')
ylabel('number of events')

% On the right-hand plot, make a histogram of rise and fall times....

subplot(1,2,2)
histogram(RiseTime,'BinWidth',Time(2)-Time(1),'FaceColor',[0.2,0.2,0.2],'EdgeColor',[0.1,0.1,0.1])
hold on
histogram(FallTime,'BinWidth',Time(2)-Time(1),'FaceColor',[0.6,0.2,0.2],'EdgeColor',[0.3,0.1,0.1])
xlabel('time (s)')
ylabel('number of events')
legend('rise time', 'fall time')

end

%%

function save_output_callback(~,~,h)

% The data is saved to three sheets of a spreadsheet. The user can choose 
% the name of the spreadsheet. For simplicity, a name related to the
% original data file is suggested as a default....

SuggestedFileName = strsplit(get(h.file_textbox, 'String'),'.');
SuggestedFileName = SuggestedFileName{1};

[FileName, FilePath] = uiputfile(...
 {'*.xlsx';'*.*'},...
 'Save or append to file....',...
 [SuggestedFileName '_peaks']);

if ~FileName
    return
end

% Go get all of the data that we wish to summarize and save....

if get(h.shutter_checkbox,'Value')
    Observations = get(h.shutter_free_table, 'Data');
else
    Observations = get(h.observation_table, 'Data');
end

SmoothedObservations = get(h.observation_table, 'UserData');
Peaks = get(h.peak_table, 'Data');
Time = get(h.time_table, 'Data');

HalfBefore = get(h.half_table, 'Data');
HalfAfter = get(h.half_table, 'UserData');

[T,N] = size(Peaks);

% If there's no time series provided, we improvise....

if length(Time) ~= T
    Time = 1:T;
end

% Pull out the peaks and the valleys....

[PeakLocation,PeakIndex] = find(Peaks);
[HalfBeforeLocation,~] = find(HalfBefore);
[HalfAfterLocation,~] = find(HalfAfter);
NumPeaks = length(PeakLocation);

% Now we put together the first sheet of data about the individual
% peaks....

PeakData = zeros(NumPeaks,7);
PeakLabels ={'Series', 'Peak Number', 'Time', 'Height', 'Rise Time (half max to max)', 'Decay Time (max to half max)', 'FWHM'};

BigCount = 1;
for ii=1:N
    for jj = 1:sum(PeakIndex==ii)
        PeakData(BigCount,1) = ii;
        PeakData(BigCount,2) = jj;
        PeakData(BigCount,3) = Time(PeakLocation(BigCount));
        PeakData(BigCount,4) = Observations(PeakLocation(BigCount),PeakIndex(BigCount))./SmoothedObservations(PeakLocation(BigCount),PeakIndex(BigCount)) - 1;
        PeakData(BigCount,5) = Time(PeakLocation(BigCount))-Time(HalfBeforeLocation(BigCount));
        PeakData(BigCount,6) = Time(HalfAfterLocation(BigCount))-Time(PeakLocation(BigCount));
        PeakData(BigCount,7) = Time(HalfAfterLocation(BigCount))-Time(HalfBeforeLocation(BigCount));
        
        BigCount = BigCount+1;
    end
end

% A second sheet collects data about the series (or ROIs)....

SeriesData = zeros(N,8);
SeriesLabels ={'Series', 'Number of Events', 'Mean Height', 'Mean Rise Time', 'Mean Decay Time', 'Mean FWHM', 'Mean InterEvent Interval', 'Mean Frequency'};

for ii = 1:N
    SeriesData(ii,1) = ii;
    Indices = find(PeakIndex==ii);
    SeriesData(ii,2) = length(Indices);
    if ~isempty(Indices)
        SeriesData(ii,3) = mean(PeakData(Indices,4));
        SeriesData(ii,4) = mean(PeakData(Indices,5));
        SeriesData(ii,5) = mean(PeakData(Indices,6));
        SeriesData(ii,6) = mean(PeakData(Indices,7));
    end
    if length(Indices) > 1
        SeriesData(ii,7) = mean(PeakData(Indices(2:end),3)-PeakData(Indices(1:end-1),3));
        SeriesData(ii,8) = 1/mean(PeakData(Indices(2:end),3)-PeakData(Indices(1:end-1),3));
    end
end

% A third sheet brings it ALL together....

SummaryData = zeros(1,8);
SummaryLabels ={'Mean Number of Signal Events', 'Standard Deviation of the Number of Signal Events', 'Mean Height (all)', 'Mean Rise Time (all)', 'Mean Decay Time (all)', 'Mean FWHM (all)', 'Mean InterEvent Interval (all)', 'Mean Frequency (all)'};
SummaryData(1) = mean(SeriesData(:,2));
SummaryData(2) = std(SeriesData(:,2));
SummaryData(3) = SeriesData(:,2)'*SeriesData(:,3)/sum(SeriesData(:,2));
SummaryData(4) = SeriesData(:,2)'*SeriesData(:,4)/sum(SeriesData(:,2));
SummaryData(5) = SeriesData(:,2)'*SeriesData(:,5)/sum(SeriesData(:,2));
SummaryData(6) = SeriesData(:,2)'*SeriesData(:,6)/sum(SeriesData(:,2));
SummaryData(7) = max(SeriesData(:,2)'-1,0)*SeriesData(:,7)/sum(max(SeriesData(:,2)'-1,0));
SummaryData(8) = 1/SummaryData(7);

% Now delete the file if it already exists....

if exist([FilePath FileName],'file')
    delete([FilePath FileName])     
end

% And write a new one!!

try
    xlswrite([FilePath FileName],[PeakLabels;num2cell(PeakData)],1)
    xlswrite([FilePath FileName],[SeriesLabels;num2cell(SeriesData)],2)
    xlswrite([FilePath FileName],[SummaryLabels;num2cell(SummaryData)],3)
catch
    fprintf(1, 'Error.\n');
    warndlg(['Existing file ' [FilePath FileName] ' may be open. Please close the file and try again.'], 'Predictors Not Saved')
    return
end

end

%%

function slider_callback(~,~,h)

% When the user clicks the slider, we update the displayed series number
% and then plot the requested series....

SeriesNumber = round(get(h.series_slider, 'Value'));

set(h.series_slider, 'Value', SeriesNumber)
set(h.series_text, 'String', SeriesNumber)

PlotSeries(h)
end

%%

% Beyond this point are the "old" PeakFinder functions, included for
% comparison's sake. These include the functions extrema() and
% relevantPeaks2() -- and are provided without any editing.

%%

function [xmax,imax,xmin,imin] = old_extrema(x)
%EXTREMA   Gets the global extrema points from a time series.
%   [XMAX,IMAX,XMIN,IMIN] = EXTREMA(X) returns the global minima and maxima
%   points of the vector X ignoring NaN's, where
%    XMAX - maxima points in descending order
%    IMAX - indexes of the XMAX
%    XMIN - minima points in descending order
%    IMIN - indexes of the XMIN
%
%   DEFINITION (from http://en.wikipedia.org/wiki/Maxima_and_minima):
%   In mathematics, maxima and minima, also known as extrema, are points in
%   the domain of a function at which the function takes a largest value
%   (maximum) or smallest value (minimum), either within a given
%   neighbourhood (local extrema) or on the function domain in its entirety
%   (global extrema).
%
%   Example:
%      x = 2*pi*linspace(-1,1);
%      y = cos(x) - 0.5 + 0.5*rand(size(x)); y(40:45) = 1.85; y(50:53)=NaN;
%      [ymax,imax,ymin,imin] = extrema(y);
%      plot(x,y,x(imax),ymax,'g.',x(imin),ymin,'r.')
%
%   See also EXTREMA2, MAX, MIN

%   Written by
%   Lic. on Physics Carlos Adrin Vargas Aguilera
%   Physical Oceanography MS candidate
%   UNIVERSIDAD DE GUADALAJARA
%   Mexico, 2004
%
%   nubeobscura@hotmail.com

% From       : http://www.mathworks.com/matlabcentral/fileexchange
% File ID    : 12275
% Submited at: 2006-09-14
% 2006-11-11 : English translation from spanish.
% 2006-11-17 : Accept NaN's.
% 2007-04-09 : Change name to MAXIMA, and definition added.


xmax = [];
imax = [];
xmin = [];
imin = [];

% Vector input?
Nt = numel(x);
if Nt ~= length(x)
    error('Entry must be a vector.')
end

% NaN's:
inan = find(isnan(x));
indx = 1:Nt;
if ~isempty(inan)
    indx(inan) = [];
    x(inan) = [];
    Nt = length(x);
end

% Difference between subsequent elements:
dx = diff(x);

% Is an horizontal line?
if ~any(dx)
    return
end

% Flat peaks? Put the middle element:
a = find(dx~=0);              % Indexes where x changes
lm = find(diff(a)~=1) + 1;    % Indexes where a do not changes
d = a(lm) - a(lm-1);          % Number of elements in the flat peak
a(lm) = a(lm) - floor(d/2);   % Save middle elements
a(end+1) = Nt;

% Peaks?
xa  = x(a);             % Serie without flat peaks
b = (diff(xa) > 0);     % 1  =>  positive slopes (minima begin)
% 0  =>  negative slopes (maxima begin)
xb  = diff(b);          % -1 =>  maxima indexes (but one)
% +1 =>  minima indexes (but one)
imax = find(xb == -1) + 1; % maxima indexes
imin = find(xb == +1) + 1; % minima indexes
imax = a(imax);
imin = a(imin);

nmaxi = length(imax);
nmini = length(imin);

% Maximum or minumim on a flat peak at the ends?
if (nmaxi==0) && (nmini==0)
    if x(1) > x(Nt)
        xmax = x(1);
        imax = indx(1);
        xmin = x(Nt);
        imin = indx(Nt);
    elseif x(1) < x(Nt)
        xmax = x(Nt);
        imax = indx(Nt);
        xmin = x(1);
        imin = indx(1);
    end
    return
end

% Maximum or minumim at the ends?
if (nmaxi==0)
    imax(1:2) = [1 Nt];
elseif (nmini==0)
    imin(1:2) = [1 Nt];
else
    if imax(1) < imin(1)
        imin(2:nmini+1) = imin;
        imin(1) = 1;
    else
        imax(2:nmaxi+1) = imax;
        imax(1) = 1;
    end
    if imax(end) > imin(end)
        imin(end+1) = Nt;
    else
        imax(end+1) = Nt;
    end
end
xmax = x(imax);
xmin = x(imin);

% NaN's:
if ~isempty(inan)
    imax = indx(imax);
    imin = indx(imin);
end

% Same size as x:
imax = reshape(imax,size(xmax));
imin = reshape(imin,size(xmin));

% Descending order:
[temp,inmax] = sort(-xmax); clear temp
xmax = xmax(inmax);
imax = imax(inmax);
[xmin,inmin] = sort(xmin);
imin = imin(inmin);


% Carlos Adrin Vargas Aguilera. nubeobscura@hotmail.com

end

%%

function [indices,Area] = old_relevantPeaks2(Pind,Vind,Data,Time)

PVind = sort([Pind,Vind]);

Area = zeros(1,length(PVind));
Data = Data-min(Data);

if Pind(1) < Vind(1)
    start = 1;
else
    start = 2;
end

for i = start:2:length(PVind)
    
    iData = Data(PVind(i));
    iTime = Time(PVind(i));
    
    j = 1;
    left = PVind(i);
    pleft = left;
    while (i-j > 0) && (Data(pleft) >= Data(left)) && (Data(PVind(i-(j-1))) <= iData)
        pleft = left;
        left = PVind(i-j);
        j = j + 2;
    end
    j = 1;
    right = PVind(i);
    pright = right;
    while (i+j <= length(PVind)) && (Data(pright) >= Data(right)) && (Data(PVind(i+(j-1))) <= iData)
        pright = right;
        right = PVind(i+j);
        j = j + 2;
    end
    
    if left == PVind(i)
        leftData = Data(right);
        leftTime = iTime - Time(right);
    else
        leftData = Data(left);
        leftTime = Time(left);
    end
    
    if right == PVind(i)
        rightData = Data(left);
        rightTime = iTime + Time(left);
    else
        rightData = Data(right);
        rightTime = Time(right);
    end
    
    sides = ([((iData-leftData)^2 + (iTime-leftTime)^2)^0.5,
        ((iData-rightData)^2 + (iTime-rightTime)^2)^0.5,
        ((rightData-leftData)^2 + (rightTime-leftTime)^2)^0.5]);
    
    perimeter = sum(sides)/2;
    Area(i) = (perimeter*(perimeter-sides(1))*(perimeter-sides(2))*(perimeter-sides(3)))^0.5;
    
end

indices = unique(PVind.*(Area >= mean(Area)+std(Area)));
indices = indices(2:end);

end